{-# LANGUAGE OverloadedStrings #-}

module Format where

import Types

import qualified Data.ByteString as B
import qualified Data.ByteString.Builder as BB
import qualified Data.ByteString.Internal as BI
import qualified Data.ByteString.Lazy as BL
import Data.String
import Substr

pprHunkHdr i j = fromString $ "@@ -" ++ show i ++ " +" ++ show j ++ " @@" 

lineSep = fromString "\n"

pprHunks :: [Hunk] -> BS
pprHunks = B.concat . map pprHunk

pprHunk :: Hunk -> BS
pprHunk ((i, j), toks) = B.concat ((pprHunkHdr i j <> lineSep) : map pprDiff1 toks)

pprDiff1 (op, (tok, s)) =
  fromString pfx <> escNewlines s <> lineSep
  where
    pfx =
      case (op, tok) of
        (Add, True) -> "+|"
        (Remove, True) -> "-|"
        (Keep, True) -> " |"
        (Add, False) -> "+."
        (Remove, False) -> "-."
        (Keep, False) -> " ."

escNewlines = BL.toStrict . BB.toLazyByteString . escNewlines'

escNewlines' s
  | B.null s = mempty
  | B.head s == BI.c2w '\n' = BB.stringUtf8 "\\n" <> escNewlines' (B.tail s)
  | B.head s == BI.c2w '\\' = BB.stringUtf8 "\\\\" <> escNewlines' (B.tail s)
  | otherwise = BB.word8 (B.head s) <> escNewlines' (B.tail s)
