module Merge
  ( MergeOpts
  , mergeOption
  ) where

import qualified Data.ByteString as B
import Data.String
import Options.Applicative
import Types

data MergeOpts =
  MergeOpts
    { mergeDoMerge :: Bool
    , mergeForceWhitespace :: Bool
    , mergeKeepWhitespace :: Bool
    , mergeCStartStr :: BS
    , mergeMineSepStr :: BS
    , mergeYourSepStr :: BS
    , mergeEndStr :: BS
    }
  deriving (Show)

marker = fromString . replicate 7

mergeOption forPatch =
  MergeOpts <$>
  switch
    (short 'm' <>
     long "merge" <>
     help
       (if forPatch
          then "Merge using conflict markers instead of printing the rejected hunks"
          else "Output the merged file instead of the patch")) <*>
  switch
    (short 'w' <>
     long "whitespace" <>
     help
       ((if forPatch
           then "Force rejecting a thunk"
           else "Force a merge conflict") ++
        " on whitespace mismatch")) <*>
  switch
    (short 'k' <>
     long "keep-whitespace" <>
     help
       ("On whitespace mismatch, default to the version from " ++
        (if forPatch
           then "original file"
           else "MYFILE") ++
        " instead of the one from " ++
        (if forPatch
           then "patch"
           else "YOURFILE"))) <*>
  strOption
    (long "merge-start" <>
     value (marker '<') <> help "Marker for the beginning of a conflict") <*>
  strOption
    (long "merge-mine" <>
     value (marker '|') <>
     help "Marker that separates `mine' from `original' part of the conflict") <*>
  strOption
    (long "merge-your" <>
     value (marker '=') <>
     help "Marker that separates `original' from `your' part of the conflict") <*>
  strOption
    (long "merge-end" <>
     value (marker '>') <> help "Marker for the end of a conflict")
